"""
Classifies: CHEBI:147334 1-phosphatidyl-1D-myo-inositol anion derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_1_phosphatidyl_1D_myo_inositol_anion_derivative(smiles: str):
    """
    Determines if a molecule is a 1-phosphatidyl-1D-myo-inositol anion derivative.
    These are phosphatidylinositol anions where the inositol is either not phosphorylated 
    or phosphorylated at position 4 and/or position 5.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a 1-phosphatidyl-1D-myo-inositol anion derivative
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for basic inositol ring with phosphatidyl linkage
    inositol_patterns = [
        Chem.MolFromSmarts("[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OCC)[C@@H](O)[C@H]1O"),
        Chem.MolFromSmarts("O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OCC)[C@H](O)[C@@H]1O")
    ]
    
    has_inositol = False
    for pattern in inositol_patterns:
        if pattern is not None and mol.HasSubstructMatch(pattern):
            has_inositol = True
            break
            
    if not has_inositol:
        return False, "No myo-inositol ring with correct stereochemistry and phosphatidyl linkage found"

    # Check for glycerol backbone with ester linkages
    glycerol_pattern = Chem.MolFromSmarts("COP([O-])(=O)OC[C@@H](COC(=O)*)OC(=O)*")
    if not mol.HasSubstructMatch(glycerol_pattern):
        glycerol_pattern_alt = Chem.MolFromSmarts("COP([O-])(=O)OC[C@H](COC(=O)*)OC(=O)*")
        if not mol.HasSubstructMatch(glycerol_pattern_alt):
            return False, "No proper glycerol backbone with ester linkages found"

    # Check for phosphate groups at positions 4 and/or 5
    phosphate_pattern = Chem.MolFromSmarts("OP([O-])([O-])=O")
    matches = mol.GetSubstructMatches(phosphate_pattern)
    
    if len(matches) == 0:
        return True, "Unphosphorylated phosphatidylinositol"
    
    # Patterns for phosphates at positions 4 and 5
    correct_phosphate_patterns = [
        Chem.MolFromSmarts("[C@H]1[C@H](O)[C@H](O)[C@@H](OP([O-])([O-])=O)[C@H](OP([O-])([O-])=O)[C@H]1O"),
        Chem.MolFromSmarts("[C@H]1[C@H](O)[C@H](O)[C@@H](OP([O-])([O-])=O)[C@H](O)[C@H]1O"),
        Chem.MolFromSmarts("[C@H]1[C@H](O)[C@H](O)[C@H](O)[C@@H](OP([O-])([O-])=O)[C@H]1O")
    ]
    
    for pattern in correct_phosphate_patterns:
        if pattern is not None and mol.HasSubstructMatch(pattern):
            return True, "Phosphatidylinositol with phosphate groups at positions 4 and/or 5"
    
    return False, "Contains phosphate groups at positions other than 4 and 5"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:147334',
                          'name': '1-phosphatidyl-1D-myo-inositol anion '
                                  'derivative',
                          'definition': 'A phosphatidylinositol anion where '
                                        'the inositol is either not '
                                        'phosphorylated or phosphorylated at '
                                        'position 4 and/or position 5; major '
                                        'species at pH 7.3.',
                          'parents': ['CHEBI:62643']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.26923076923076916 is too low.\n'
               'True positives: '
               "[('CCCCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)OC(=O)CCCCCCC\\\\C=C/CCCCCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O)[C@H](O)[C@@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@@H]1[C@H](O)[C@H](O)[C@@H](OP([O-])([O-])=O)[C@H](OP([O-])([O-])=O)[C@H]1O)OC(=O)CCCCCCC', "
               "'Phosphatidylinositol with phosphate groups at positions 4 "
               "and/or 5'), "
               "('CCCCC\\\\C=C/C\\\\C=C/CCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)OC(=O)CCCCCCC\\\\C=C/C\\\\C=C/CCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O)[C@H](O)[C@@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCC\\\\C=C/CCCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)OC(=O)CCC\\\\C=C/C\\\\C=C/C\\\\C=C/C\\\\C=C/CCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)OC(=O)CCCCCCCCCCC', "
               "'Unphosphorylated phosphatidylinositol')]\n"
               'False positives: '
               "[('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O[C@H]2O[C@@H]([C@H]([C@@H]([C@H]2NC(C)=O)O)O)CO)O)O)O)O)OCC(COC(CCCCCCCCCCCCCCCCC)=O)OC(=O)CCCCCCC/C=C\\\\CCCCCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CC(=O)N[C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@@H]1O[C@@H]1[C@@H](O)[C@H](O)[C@@H](O)[C@@H](O)[C@H]1OP([O-])(=O)OC[C@@H](COC([*])=O)OC([*])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O(C(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)C[C@H](COP(O[C@H]1[C@@H]([C@H]([C@@H]([C@H]([C@H]1O)O)O)O)O)(=O)[O-])O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])([O-])=O)[C@@H](OP([O-])([O-])=O)[C@H]1O)OC(=O)CCCCCCCCCCCCCCC', "
               "'Phosphatidylinositol with phosphate groups at positions 4 "
               "and/or 5'), "
               "('CCCCCC\\\\C=C/CCCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)CO)O)O)O)OP(OC[C@@H](COC(=O)*)OC(=O)*)(=O)[O-])O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO)O)O)O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('OC1C(O)C(O)C(OP([O-])(=O)OCC(COC=C[*])OC([*])=O)C(O)C1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('P(OC1[C@H](O)[C@H](O)C(O)[C@H](O)[C@H]1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)([O-])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('OC(COC([*])=O)COP([O-])(=O)OC1C(O)C(O)C(O)C(O)C1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O[C@H]2O[C@@H]([C@H]([C@@H]([C@H]2[NH3+])O)O)CO)O)O)O)O)OCC(COC(CCCCCCCCCCCCCCCCC)=O)OC(=O)CCCCCCC/C=C\\\\CCCCCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCC\\\\C=C/CCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O)O)O)O)O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)CO)O)O)O)OC[C@@H](COC(*)=O)OC(*)=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O[C@H](CO\\\\C=C/[*])COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCCCCCCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CC(C)CCC[C@@H](C)CCC[C@@H](C)CCC[C@@H](C)CCOC[C@@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O)OCC[C@H](C)CCC[C@H](C)CCC[C@H](C)CCCC(C)C', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[H][C@@](COC([*])=O)(COP([O-])(=O)O[C@@H]1[C@H](O)[C@H](O)[C@@H](O)[C@H](O)[C@H]1O[C@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1[NH3+])OC([*])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O*)O*)O)O)OP(OC[C@@H](COC(=O)*)OC(=O)*)(=O)[O-])O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)COC(=O)*)O)O)O)OP(OC[C@@H](COC(=O)*)OC(=O)*)(=O)[O-])O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO)O)O)O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('P(OC1[C@H](O)[C@@H](O)C(O)[C@@H](O)[C@H]1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCC/C=C\\\\CCCCCC)([O-])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('P(OC1[C@H](O)[C@@H](O)C(O)[C@@H](O)[C@H]1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCC/C=C\\\\CCCCCCCC)([O-])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OC[C@@H](COC([*])=O)O[*])[C@H](O)[C@@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O([C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)COC(=O)CCCCCCCCCCCCCCC)O)O)O)OC(CCCCCCCCCCCCCCC)=O)O)O)O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO)O)O)O)P(=O)([O-])OC[C@@H](COC(CCCCCCCC[C@H](CCCCCCCC)C)=O)OC(CCCCCCCCCCCCCCC)=O.[Na+]', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)O)OP(OC[C@@H](COC=C*)OC(*)=O)(=O)[O-])O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCCCCCCCCCCCCCCCCCCC(O)C(=O)N[C@@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O[C@@H]2O[C@H](COP([O-])(=O)O[C@H]3[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]3O)[C@@H](O)[C@H](O)[C@@H]2O)[C@H]1O)[C@H](O)[C@H](O)CCCCCCCCCCCCCC', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('P(OC1[C@H](O)[C@@H](O)C(O)[C@@H](O)[C@H]1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\C/C=C\\\\CC)COC(=O)CCCCCCCCCCCCCCCCC)([O-])=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[H][C@@](COC([*])=O)(COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])([O-])=O)[C@@H](OP([O-])([O-])=O)[C@H]1O)OC([*])=O', "
               "'Phosphatidylinositol with phosphate groups at positions 4 "
               "and/or 5'), "
               "('OC1C(O)C(O)C(OP([O-])(=O)OCC(COC([*])=O)OC([*])=O)C(O)C1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O([C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)COC(=O)CCCCCCCCCCCCCCC)O)O)O)OC(CCCCCCCCCCCCCCC)=O)O)O)O[C@@H]3[C@H]([C@H]([C@@H]([C@H](O3)CO)O)O)O)P(=O)([O-])OC[C@@H](COC(CCCCCCCC[C@H](CCCCCCCC)C)=O)OC(CCCCCCCCCCCCCCC)=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('OC(COC=C[*])COP([O-])(=O)OC1C(O)C(O)C(O)C(O)C1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])(=O)OC[C@@H](CO\\\\C=C/[*])OC([*])=O)[C@H](O)[C@@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('C(C/C=C\\\\C/C=C\\\\C/C=C\\\\C/C=C\\\\CCCCC)CC(=O)OC[C@H](COP(O[C@H]1[C@@H]([C@H]([C@@H]([C@H]([C@H]1O)O)O)O)O)(=O)[O-])O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('O=P([O-])(O[C@@H]1[C@@H]([C@@H]([C@H]([C@@H]([C@H]1O)O)O)O)O[C@@H]2[C@H]([C@H]([C@@H]([C@H](O2)COC(*)=O)O)O)O)OC[C@@H](COC(*)=O)OC(*)=O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCCCCCCCCCC(=O)OC[C@@H](O)COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('CCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](OP([O-])([O-])=O)[C@@H](OP([O-])([O-])=O)[C@H]1O)OC(=O)CCCCCCC', "
               "'Phosphatidylinositol with phosphate groups at positions 4 "
               "and/or 5'), "
               "('O[C@H](COC([*])=O)COP([O-])(=O)O[C@H]1[C@H](O)[C@@H](O)[C@H](O)[C@@H](O)[C@H]1O', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('P(OC1C(O)C(O)C(O)[C@@H](O)C1O)(OC[C@H](OC(=O)CCCCCCC/C=C\\\\CCCC)COC(=O)CCCCCCCCCCCCCCCC)([O-])=O.[NH4+]', "
               "'Unphosphorylated phosphatidylinositol'), "
               "('[C@@H]1([C@@H]([C@@H]([C@@H]([C@H]([C@@H]1O)O)O)OC(=O)*)OP(OC[C@H](COC(=O)*)OC(=O)*)(=O)[O-])O[C@@H]2[C@H]([NH3+])[C@@H](O)[C@@H]([C@H](O2)CO)O', "
               "'Unphosphorylated phosphatidylinositol')]\n"
               'False negatives: '
               "[('CCCCCCCC(=O)OC[C@H](COP([O-])(=O)O[C@@H]1[C@H](O)[C@H](O)[C@@H](OP([O-])([O-])=O)[C@H](O)[C@H]1O)OC(=O)CCCCCCC', "
               "'Contains phosphate groups at positions other than 4 and 5')]",
    'attempt': 4,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 43,
    'num_true_negatives': 183848,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.1568627450980392,
    'recall': 1.0,
    'f1': 0.2711864406779661,
    'accuracy': 0.9997661759987819}


"""
Classifies: CHEBI:23643 depsipeptide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors


def is_depsipeptide(smiles: str):
    """
    Determines if a molecule is a depsipeptide based on its SMILES string.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a depsipeptide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Look for ester bonds (-O-C(=O)-)
    ester_pattern = Chem.MolFromSmarts('[#6]-O-C(=O)-[#6]')
    ester_matches = mol.GetSubstructMatches(ester_pattern)
    
    # Look for amide bonds (-N-C(=O)-)
    amide_pattern = Chem.MolFromSmarts('[#7]-C(=O)-[#6]')
    amide_matches = mol.GetSubstructMatches(amide_pattern)
    
    if not ester_matches:
        return False, "No ester bonds found"
    
    if not amide_matches:
        return False, "No amide bonds found"
    
    # Look for cyclic structure
    ring_info = mol.GetRingInfo()
    if not ring_info.NumRings():
        return False, "No cyclic structure found"
        
    # Count number of ester and amide bonds
    num_esters = len(ester_matches)
    num_amides = len(amide_matches)
    
    # Check for amino acid residues
    amino_acid_pattern = Chem.MolFromSmarts('[NX3,NX4+][CH]([CH2,CH3,CH,C])[C](=[O])[O,N]')
    amino_acid_matches = mol.GetSubstructMatches(amino_acid_pattern)
    
    # Check for hydroxy acid residues
    hydroxy_acid_pattern = Chem.MolFromSmarts('[OH][CH2,CH]([CH2,CH3,CH,C])[C](=[O])[O,N]')
    hydroxy_acid_matches = mol.GetSubstructMatches(hydroxy_acid_pattern)
    
    if not amino_acid_matches and not hydroxy_acid_matches:
        return False, "No amino acid or hydroxy acid residues found"
    
    if num_esters > 0 and num_amides > 0 and (amino_acid_matches or hydroxy_acid_matches):
        return True, f"Contains {num_esters} ester bonds, {num_amides} amide bonds, and amino/hydroxy acid residues in a cyclic structure"
    
    return False, "Does not meet all criteria for depsipeptide classification"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23643',
                          'name': 'depsipeptide',
                          'definition': 'A natural or synthetic compound '
                                        'having a sequence of amino and '
                                        'hydroxy carboxylic acid residues '
                                        '(usually alpha-amino and '
                                        'alpha-hydroxy acids), commonly but '
                                        'not necessarily regularly '
                                        'alternating.',
                          'parents': ['CHEBI:16670']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 156,
    'num_false_positives': 100,
    'num_true_negatives': 12852,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.609375,
    'recall': 0.9811320754716981,
    'f1': 0.7518072289156627,
    'accuracy': 0.9921440012203493}


"""
Classifies: CHEBI:25716 organothiophosphorus compound
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_organothiophosphorus_compound(smiles: str):
    """
    Determines if a molecule is an organothiophosphorus compound (contains P-S bond).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an organothiophosphorus compound, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check if molecule contains phosphorus
    has_p = False
    p_atoms = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'P':
            has_p = True
            p_atoms.append(atom)
    
    if not has_p:
        return False, "No phosphorus atoms found"

    # Check for P-S bonds
    p_s_bonds = []
    for p_atom in p_atoms:
        for neighbor in p_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'S':
                bond = mol.GetBondBetweenAtoms(p_atom.GetIdx(), neighbor.GetIdx())
                if bond is not None:
                    p_s_bonds.append(bond)

    if not p_s_bonds:
        return False, "No phosphorus-sulfur bonds found"

    # Check for carbon atoms (to confirm it's an organo compound)
    has_carbon = False
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            has_carbon = True
            break
            
    if not has_carbon:
        return False, "No carbon atoms found - not an organo compound"

    # Count number of P-S bonds
    num_ps_bonds = len(p_s_bonds)
    
    return True, f"Found {num_ps_bonds} phosphorus-sulfur bond(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25716',
                          'name': 'organothiophosphorus compound',
                          'definition': 'An organothiophosphorus compound is '
                                        'an organophosphorus compound which '
                                        'contains a phosphorus-sulfur bond.',
                          'parents': ['CHEBI:25710', 'CHEBI:26835']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 9,
    'num_false_positives': 66,
    'num_true_negatives': 183733,
    'num_false_negatives': 4,
    'num_negatives': None,
    'precision': 0.12,
    'recall': 0.6923076923076923,
    'f1': 0.20454545454545456,
    'accuracy': 0.999619176114726}


"""
Classifies: CHEBI:140332 4'-methoxyflavanones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_4__methoxyflavanones(smiles: str):
    """
    Determines if a molecule is a 4'-methoxyflavanone (methoxyflavanone with methoxy at position 4').
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a 4'-methoxyflavanone, False otherwise
        str: Reason for classification
    """
    # Check if SMILES is valid
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define flavanone core SMARTS pattern
    # Matches the basic flavanone skeleton with B-ring (4' position) marked with [#0]
    flavanone_pattern = Chem.MolFromSmarts('[O;H0]1[c;H0]2[c;H0]([cH,$(c(C))]([cH,$(c(C))])([cH,$(c(C))])[cH,$(c(C))])C(=O)C[CH]([#0]3[cH,$(c(C))][cH,$(c(C))][cH,$(c(C))][cH,$(c(C))][cH,$(c(C))]3)O2')
    
    # Define methoxy group SMARTS pattern
    methoxy_pattern = Chem.MolFromSmarts('OC')

    # Check if molecule contains flavanone core
    matches = mol.GetSubstructMatches(flavanone_pattern)
    if not matches:
        return False, "Not a flavanone structure"

    # For each flavanone core match found
    for match in matches:
        # Get the atom index corresponding to the 4' position (marked with [#0] in SMARTS)
        b_ring_pos = None
        for i, idx in enumerate(match):
            atom = mol.GetAtomWithIdx(idx)
            if atom.GetAtomicNum() == 0:  # Dummy atom in SMARTS
                b_ring_pos = idx
                break
                
        if b_ring_pos is None:
            continue

        # Find all methoxy groups
        methoxy_matches = mol.GetSubstructMatches(methoxy_pattern)
        
        # Check if any methoxy group is attached to 4' position
        for methoxy_match in methoxy_matches:
            # Get the oxygen atom of methoxy group
            o_atom = mol.GetAtomWithIdx(methoxy_match[0])
            
            # Check if this methoxy is attached to the 4' position
            for neighbor in o_atom.GetNeighbors():
                if neighbor.GetIdx() == b_ring_pos:
                    return True, "Found methoxy group at 4' position of flavanone"

    return False, "No methoxy group found at 4' position of flavanone"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:140332',
                          'name': "4'-methoxyflavanones",
                          'definition': 'Any methoxyflavanone having a methoxy '
                                        "substituent located at position 4'.",
                          'parents': ['CHEBI:25240']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.GetSubstructMatches(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    GetSubstructMatches(RDKit::ROMol self, RDKit::MolBundle '
             'query, RDKit::SubstructMatchParameters params)\n'
             '    GetSubstructMatches(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    GetSubstructMatches(RDKit::ROMol self, RDKit::MolBundle '
             'query, bool uniquify=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False, unsigned int maxMatches=1000)\n'
             '    GetSubstructMatches(RDKit::ROMol self, RDKit::ROMol query, '
             'bool uniquify=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False, unsigned int maxMatches=1000)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:24751 hydroxytoluene
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_hydroxytoluene(smiles: str):
    """
    Determines if a molecule is a hydroxytoluene (toluene with one or more OH groups).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxytoluene, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Find aromatic rings
    rings = mol.GetRingInfo()
    aromatic_rings = []
    for ring in rings.AtomRings():
        if len(ring) == 6:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if all(atom.GetIsAromatic() for atom in atoms):
                aromatic_rings.append(ring)

    if not aromatic_rings:
        return False, "No aromatic 6-membered rings found"

    # Check for benzene ring with all carbons
    has_valid_ring = False
    for ring in aromatic_rings:
        atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if all(atom.GetSymbol() == 'C' for atom in atoms):
            has_valid_ring = True
            ring_atoms = set(ring)
            break

    if not has_valid_ring:
        return False, "No benzene ring found"

    # Look for methyl group attached to ring
    has_methyl = False
    for atom_idx in ring_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in ring_atoms:
                if neighbor.GetSymbol() == 'C':
                    # Check if it's a methyl group by counting hydrogens
                    if neighbor.GetTotalNumHs() == 3:
                        has_methyl = True
                        break
    
    if not has_methyl:
        return False, "No methyl group found attached to ring"

    # Look for hydroxyl groups attached to ring
    hydroxy_count = 0
    for atom_idx in ring_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        for neighbor in atom.GetNeighbors():
            if neighbor.GetIdx() not in ring_atoms:
                if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                    hydroxy_count += 1

    if hydroxy_count == 0:
        return False, "No hydroxyl groups found"

    return True, f"Found toluene with {hydroxy_count} hydroxyl group(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:24751',
                          'name': 'hydroxytoluene',
                          'definition': 'Any member of the class of  toluenes '
                                        'carrying one or more hydroxy '
                                        'substituents.',
                          'parents': ['CHEBI:27024', 'CHEBI:33853']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: 'Atom' object has no attribute 'GetIsImplicit'",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 100,
    'num_true_negatives': 5959,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.047619047619047616,
    'recall': 1.0,
    'f1': 0.0909090909090909,
    'accuracy': 0.9835092348284961}


"""
Classifies: CHEBI:13643 glycol
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_glycol(smiles: str):
    """
    Determines if a molecule is a glycol (diol with hydroxy groups on different carbons).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a glycol, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Find all OH groups
    oh_pattern = Chem.MolFromSmarts("[OH]")
    oh_matches = mol.GetSubstructMatches(oh_pattern)
    
    if len(oh_matches) < 2:
        return False, "Less than 2 OH groups found"
        
    # Get carbons attached to OH groups
    oh_carbons = set()
    for match in oh_matches:
        oh_oxygen = mol.GetAtomWithIdx(match[0])
        for neighbor in oh_oxygen.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                oh_carbons.add(neighbor.GetIdx())
                
    # Check if OH groups are on different carbons
    if len(oh_carbons) < 2:
        return False, "OH groups are on the same carbon"
        
    # Find shortest path between OH carbons to check if they are adjacent
    paths = []
    oh_carbons_list = list(oh_carbons)
    for i in range(len(oh_carbons_list)):
        for j in range(i+1, len(oh_carbons_list)):
            path = Chem.GetShortestPath(mol, oh_carbons_list[i], oh_carbons_list[j])
            if path:
                paths.append(len(path)-1)
    
    min_distance = min(paths) if paths else 0
    
    if min_distance == 1:
        return True, "Glycol with adjacent OH groups"
    elif min_distance > 1:
        return True, f"Glycol with OH groups separated by {min_distance-1} carbon(s)"
    else:
        return False, "OH groups are not properly connected"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:13643',
                          'name': 'glycol',
                          'definition': 'A diol in which the two hydroxy '
                                        'groups are on different carbon atoms, '
                                        'usually but not necessarily adjacent.',
                          'parents': ['CHEBI:23824']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: cannot import name 'rdDecomposition' from "
               "'rdkit.Chem' "
               '(/Users/cjm/Library/Caches/pypoetry/virtualenvs/c3p-93U7KWO_-py3.11/lib/python3.11/site-packages/rdkit/Chem/__init__.py)',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 10,
    'num_false_positives': 100,
    'num_true_negatives': 150,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.09090909090909091,
    'recall': 1.0,
    'f1': 0.16666666666666669,
    'accuracy': 0.6153846153846154}


"""
Classifies: CHEBI:23867 dodecenoic acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors

def is_dodecenoic_acid(smiles: str):
    """
    Determines if a molecule is a dodecenoic acid, which is a C12, straight-chain fatty acid carrying a double bond at any position.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if the molecule is a dodecenoic acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule has 12 carbon atoms
    if Descriptors.HeavyAtomCount(mol) != 13:  # 12 carbon atoms + 1 oxygen atom in the carboxylic acid group
        return False, "The molecule does not have 12 carbon atoms"

    # Check if the molecule has a carboxylic acid group
    carboxyl_smarts = Chem.MolFromSmarts('C(=O)O')
    if not mol.HasSubstructMatch(carboxyl_smarts):
        return False, "The molecule does not have a carboxylic acid group"

    # Check if the molecule has a double bond
    double_bond_smarts = Chem.MolFromSmarts('C=C')
    if not mol.HasSubstructMatch(double_bond_smarts):
        return False, "The molecule does not have a double bond"

    # Check if the molecule is a straight chain
    AllChem.EmbedMolecule(mol)
    AllChem.MMFFOptimizeMolecule(mol)
    conf = mol.GetConformer()
    atoms = [conf.GetAtomPosition(i) for i in range(mol.GetNumAtoms())]
    distances = [(atoms[i] - atoms[i+1]).Length() for i in range(len(atoms)-1)]
    if not all(d > 1.3 and d < 1.7 for d in distances):
        return False, "The molecule is not a straight chain"

    return True, "The molecule is a dodecenoic acid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23867',
                          'name': 'dodecenoic acid',
                          'definition': 'A C12, straight-chain fatty acid '
                                        'carrying a double bond at any '
                                        'position.',
                          'parents': [   'CHEBI:25413',
                                         'CHEBI:59202',
                                         'CHEBI:59554']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183920,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0,
    'accuracy': 0.999994562882977}


"""
Classifies: CHEBI:27325 xanthophyll
"""
from rdkit import Chem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_xanthophyll(smiles: str):
    """
    Determines if a molecule is a xanthophyll, a subclass of carotenoids consisting of the oxygenated carotenes.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a xanthophyll, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains at least one oxygen atom
    if not any(atom.GetSymbol() == 'O' for atom in mol.GetAtoms()):
        return False, "No oxygen atoms found"

    # Check if the molecule is a carotenoid
    num_rings = rdMolDescriptors.CalcNumRings(mol)
    num_aromatic_rings = rdMolDescriptors.CalcNumAromaticRings(mol)
    if num_rings > 0 or num_aromatic_rings > 0:
        return False, "Contains rings, not a carotenoid"

    num_carbon_atoms = len([atom for atom in mol.GetAtoms() if atom.GetSymbol() == 'C'])
    if num_carbon_atoms < 20:
        return False, "Less than 20 carbon atoms, not a carotenoid"

    num_conjugated_double_bonds = Descriptors.NumAllConjonedDoubleBondsDescriptor(mol)
    if num_conjugated_double_bonds < 7:
        return False, "Less than 7 conjugated double bonds, not a carotenoid"

    return True, "Molecule is a xanthophyll, an oxygenated carotenoid"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:27325',
                          'name': 'xanthophyll',
                          'definition': 'A subclass of carotenoids consisting '
                                        'of the oxygenated carotenes.',
                          'parents': ['CHEBI:23044']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem.Descriptors' has no attribute "
             "'NumAllConjonedDoubleBondsDescriptor'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:23855 divalent carboacyl group
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_divalent_carboacyl_group(smiles: str):
    """
    Determines if a molecule contains a divalent carboacyl group.
    A divalent carboacyl group is formed by loss of OH from two carboxy groups 
    of a polycarboxylic acid.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule contains a divalent carboacyl group, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for C(=O)* pattern (carbonyl with attachment point)
    carbonyl_pattern = Chem.MolFromSmarts('C(=O)*')
    matches = mol.GetSubstructMatches(carbonyl_pattern)
    
    if len(matches) < 2:
        return False, "Does not contain at least 2 carbonyls with attachment points"

    # Find carbonyls that are connected through a carbon chain
    carbonyl_carbons = [match[0] for match in matches]
    
    # Create a copy of the molecule for path finding
    mol_copy = Chem.Mol(mol)
    
    for i in range(len(carbonyl_carbons)):
        for j in range(i+1, len(carbonyl_carbons)):
            if carbonyl_carbons[i] == carbonyl_carbons[j]:
                continue
                
            try:
                # Find shortest path between carbonyls
                path = Chem.GetShortestPath(mol_copy, carbonyl_carbons[i], carbonyl_carbons[j])
                if path and len(path) >= 2:  # Path must have at least 2 atoms
                    # Check if path contains only carbons (excluding the carbonyl carbons)
                    path_atoms = [mol.GetAtomWithIdx(idx) for idx in path[1:-1]]
                    if all(atom.GetSymbol() == 'C' for atom in path_atoms):
                        return True, f"Contains divalent carboacyl group with {len(path)-1} atoms in connecting chain"
            except:
                continue

    return False, "No carbonyls connected through carbon chain"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23855',
                          'name': 'divalent carboacyl group',
                          'definition': 'A divalent carboacyl group is a group '
                                        'formed by loss of OH from two carboxy '
                                        'groups of a polycarboxylic acid.',
                          'parents': ['CHEBI:37838']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Invariant Violation\n'
               '\t\n'
               '\tViolation occurred on line 341 in file '
               'Code/GraphMol/Matrices.cpp\n'
               '\tFailed Expression: aid1 != aid2\n'
               '\tRDKIT: 2024.03.6\n'
               '\tBOOST: 1_85\n',
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 4,
    'num_false_positives': 100,
    'num_true_negatives': 561,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.038461538461538464,
    'recall': 0.5714285714285714,
    'f1': 0.07207207207207209,
    'accuracy': 0.8458083832335329}


"""
Classifies: CHEBI:17810 1-O-(alk-1-enyl)-2-O-acyl-sn-glycero-3-phosphocholine
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_1_O__alk_1_enyl__2_O_acyl_sn_glycero_3_phosphocholine(smiles: str):
    """
    Determines if a molecule is a 1-O-(alk-1-enyl)-2-O-acyl-sn-glycero-3-phosphocholine.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule matches the class, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for phosphocholine group
    phos_pattern = Chem.MolFromSmarts('[O][P](=O)([O-])OCC[N+](C)(C)C')
    if not mol.HasSubstructMatch(phos_pattern):
        return False, "Missing phosphocholine group"

    # Check for glycerol backbone with correct stereochemistry
    glycerol_pattern = Chem.MolFromSmarts('[C@@H](COP([O-])(=O)*)(*)*')
    if not mol.HasSubstructMatch(glycerol_pattern):
        return False, "Missing glycerol backbone with correct stereochemistry"

    # Check for alk-1-enyl group at position 1
    alkenyl_pattern = Chem.MolFromSmarts('O/C=C/*')
    if not mol.HasSubstructMatch(alkenyl_pattern):
        return False, "Missing alk-1-enyl group at position 1"

    # Check for acyl group at position 2
    acyl_pattern = Chem.MolFromSmarts('C(=O)O[C@H]')
    if not mol.HasSubstructMatch(acyl_pattern):
        return False, "Missing acyl group at position 2"

    # If all patterns match, this is a valid 1-O-(alk-1-enyl)-2-O-acyl-sn-glycero-3-phosphocholine
    return True, "Contains required 1-O-(alk-1-enyl)-2-O-acyl-sn-glycero-3-phosphocholine structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17810',
                          'name': '1-O-(alk-1-enyl)-2-O-acyl-sn-glycero-3-phosphocholine',
                          'definition': 'A glycero-3-phosphocholine compound '
                                        'having an alk-1-enyl substituent at '
                                        'the 1-position and an acyl '
                                        'substituent at the 2-position.',
                          'parents': [   'CHEBI:167497',
                                         'CHEBI:68489',
                                         'CHEBI:76170',
                                         'CHEBI:78188']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 6,
    'num_false_positives': 82,
    'num_true_negatives': 183826,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.06818181818181818,
    'recall': 1.0,
    'f1': 0.1276595744680851,
    'accuracy': 0.9995541394347358}


"""
Classifies: CHEBI:131871 hydroxy polyunsaturated fatty acid anion
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_hydroxy_polyunsaturated_fatty_acid_anion(smiles: str):
    """
    Determines if a molecule is a hydroxy polyunsaturated fatty acid anion.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a hydroxy polyunsaturated fatty acid anion, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"
        
    # Check for carboxylate anion
    carboxylate = False
    for atom in mol.GetAtoms():
        if atom.GetFormalCharge() == -1 and atom.GetSymbol() == 'O':
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C':
                    for n2 in neighbor.GetNeighbors():
                        if n2.GetSymbol() == 'O' and n2.GetFormalCharge() == 0:
                            carboxylate = True
                            
    if not carboxylate:
        return False, "No carboxylate anion group found"
        
    # Check for hydroxy groups
    hydroxy_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'O' and atom.GetTotalNumHs() == 1:
            for neighbor in atom.GetNeighbors():
                if neighbor.GetSymbol() == 'C':
                    hydroxy_count += 1
                    
    if hydroxy_count == 0:
        return False, "No hydroxy groups found"
        
    # Count double bonds
    double_bond_count = 0
    for bond in mol.GetBonds():
        if bond.GetBondType() == Chem.rdchem.BondType.DOUBLE:
            # Don't count the carboxylate double bond
            atoms = [bond.GetBeginAtom(), bond.GetEndAtom()]
            if not any(a.GetFormalCharge() == -1 for a in atoms):
                double_bond_count += 1
                
    if double_bond_count < 2:
        return False, "Not polyunsaturated (less than 2 double bonds)"
        
    # Check carbon chain length (should be medium-long chain fatty acid)
    carbon_count = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if carbon_count < 12:
        return False, "Carbon chain too short for fatty acid"
        
    return True, f"Hydroxy polyunsaturated fatty acid anion with {hydroxy_count} OH group(s) and {double_bond_count} double bond(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131871',
                          'name': 'hydroxy polyunsaturated fatty acid anion',
                          'definition': 'Any polyunsaturated fatty acid anion '
                                        'carrying one or more hydroxy '
                                        'substituents.',
                          'parents': ['CHEBI:59835', 'CHEBI:76567']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 11,
    'num_false_positives': 100,
    'num_true_negatives': 7229,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0990990990990991,
    'recall': 0.9166666666666666,
    'f1': 0.17886178861788618,
    'accuracy': 0.9862416564500749}


"""
Classifies: CHEBI:22300 aldonic acid phosphate
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_aldonic_acid_phosphate(smiles: str):
    """
    Determines if a molecule is an aldonic acid phosphate.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an aldonic acid phosphate, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)O')
    if not mol.HasSubstructMatch(carboxylic_pattern):
        return False, "No carboxylic acid group found"

    # Check for phosphate group
    phosphate_pattern = Chem.MolFromSmarts('P(=O)(O)(O)O')
    if not mol.HasSubstructMatch(phosphate_pattern):
        return False, "No phosphate group found"

    # Check for hydroxy groups
    hydroxy_pattern = Chem.MolFromSmarts('OH')
    if not mol.HasSubstructMatch(hydroxy_pattern):
        return False, "No hydroxy groups found"

    # Check carbon chain between COOH and phosphate
    # First get the indices of the C in COOH and P atom
    cooh_matches = mol.GetSubstructMatches(carboxylic_pattern)
    phos_matches = mol.GetSubstructMatches(phosphate_pattern)
    
    if not cooh_matches or not phos_matches:
        return False, "Could not find required groups"

    cooh_c = cooh_matches[0][0]  # Carbon of COOH
    phos = phos_matches[0][0]    # Phosphorus atom

    # Check if there is a carbon chain path between COOH and phosphate
    path_exists = False
    for path in AllChem.FindAllPathsOfLengthN(mol, 10, useBonds=False):
        if cooh_c in path and phos in path:
            # Check if path contains carbons with OH groups
            for atom_idx in path:
                atom = mol.GetAtomWithIdx(atom_idx)
                if atom.GetSymbol() == 'C':
                    for neighbor in atom.GetNeighbors():
                        if neighbor.GetSymbol() == 'O' and neighbor.GetTotalNumHs() == 1:
                            path_exists = True
                            break
    
    if not path_exists:
        return False, "No suitable carbon chain path found between carboxylic acid and phosphate groups"

    return True, "Molecule contains carboxylic acid group, phosphate group, and hydroxy groups in appropriate arrangement"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22300',
                          'name': 'aldonic acid phosphate',
                          'definition': 'An aldonic acid in which at least one '
                                        'of the hydroxy groups has been '
                                        'phosphorylated.',
                          'parents': ['CHEBI:26816']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': 'Python argument types in\n'
             '    Mol.HasSubstructMatch(Mol, NoneType)\n'
             'did not match C++ signature:\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'RDKit::SubstructMatchParameters params=True)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'RDKit::SubstructMatchParameters params)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)\n'
             '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
             'bool recursionPossible=True, bool useChirality=False, bool '
             'useQueryQueryMatches=False)',
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:16389 ubiquinones
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_ubiquinones(smiles: str):
    """
    Determines if a molecule is a ubiquinone based on structural features.
    
    Args:
        smiles (str): SMILES string of the molecule
    
    Returns:
        bool: True if molecule is a ubiquinone, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for benzoquinone core with specific substitution pattern
    # SMARTS pattern for 2,3-dimethoxy/hydroxy-5-methyl-1,4-benzoquinone core
    core_pattern = Chem.MolFromSmarts('[$(C1(=O)C(=[C](C)C(=O)C(O[CH3,H])=C1O[CH3,H]))]')
    
    if core_pattern is None:
        return None, "Invalid SMARTS pattern"

    if not mol.HasSubstructMatch(core_pattern):
        return False, "Missing required benzoquinone core structure"

    # Check for methoxy/hydroxy groups at positions 2,3
    if not (('COC1=C(OC)' in smiles) or ('COC1=C(O)' in smiles)):
        return False, "Missing required methoxy/hydroxy groups at positions 2,3"
        
    # Check for polyprenoid side chain
    isoprenoid_pattern = Chem.MolFromSmarts('CC(=C)CC')
    if isoprenoid_pattern is None:
        return None, "Invalid isoprenoid SMARTS pattern"
        
    matches = mol.GetSubstructMatches(isoprenoid_pattern)
    
    if len(matches) < 1:
        return False, "Missing polyprenoid side chain"

    # Count isoprenoid units
    num_units = len(matches)

    if 'COC1=C(OC)' in smiles:
        return True, f"Ubiquinone with {num_units} isoprenoid units"
    else:
        return True, f"Demethylubiquinone with {num_units} isoprenoid units"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:16389',
                          'name': 'ubiquinones',
                          'definition': 'Any benzoquinone derived from '
                                        '2,3-dimethoxy-5-methylbenzoquinone; '
                                        'one of a group of naturally occurring '
                                        'homologues. The redox-active quinoid '
                                        'moiety usually carries a polyprenoid '
                                        'side chain at position 6, the number '
                                        'of isoprenoid units in which is '
                                        'species-specific. Ubiquinones are '
                                        'involved in the control of '
                                        'mitochondrial electron transport, and '
                                        'are also potent anti-oxidants.',
                          'parents': ['CHEBI:132124', 'CHEBI:26255']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': 'Attempt failed: Python argument types in\n'
               '    Mol.HasSubstructMatch(Mol, NoneType)\n'
               'did not match C++ signature:\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, RDKit::SubstructMatchParameters params=True)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'RDKit::SubstructMatchParameters params)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::MolBundle '
               'query, bool recursionPossible=True, bool useChirality=False, '
               'bool useQueryQueryMatches=False)\n'
               '    HasSubstructMatch(RDKit::ROMol self, RDKit::ROMol query, '
               'bool recursionPossible=True, bool useChirality=False, bool '
               'useQueryQueryMatches=False)',
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 1,
    'num_false_positives': 2,
    'num_true_negatives': 183909,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.3333333333333333,
    'recall': 0.5,
    'f1': 0.4,
    'accuracy': 0.9999836879394062}


"""
Classifies: CHEBI:17102 phosphoramide
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_phosphoramide(smiles: str):
    """
    Determines if a molecule is a phosphoramide - a compound where one or more OH groups
    of phosphoric acid have been replaced with amino or substituted amino groups.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a phosphoramide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Define SMARTS patterns for phosphoramide groups
    # P(=O) with at least one N attached and other substituents
    phosphoramide_patterns = [
        "[P;X4](=[O;X1])([N;X3])(~[*])(~[*])",  # General phosphoramide pattern
        "[P;X4](=[O;X1])([N;X3])([N;X3])(~[*])", # Two N atoms
        "[P;X4](=[O;X1])([N;X3])([N;X3])([N;X3])" # Three N atoms (phosphoric triamide)
    ]

    # Check for any matching patterns
    for pattern in phosphoramide_patterns:
        matches = mol.GetSubstructMatches(Chem.MolFromSmarts(pattern))
        if matches:
            # Count N atoms connected to P
            p_atom = mol.GetAtomWithIdx(matches[0][0])
            n_count = sum(1 for neighbor in p_atom.GetNeighbors() if neighbor.GetAtomicNum() == 7)
            
            if n_count == 3:
                return True, "Phosphoric triamide P(=O)(NR2)3"
            else:
                return True, f"Phosphoramide with {n_count} N atoms"

    # Check if molecule contains P but no phosphoramide group
    phosphorus_atoms = [atom for atom in mol.GetAtoms() if atom.GetAtomicNum() == 15]
    if phosphorus_atoms:
        return False, "Contains phosphorus but no phosphoramide group"
    
    return False, "No phosphorus atoms found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17102',
                          'name': 'phosphoramide',
                          'definition': 'A compound in which one or more of '
                                        'the OH groups of phosphoric acid have '
                                        'been replaced with an amino or '
                                        'substituted amino group. The term is '
                                        'commonly confined to the phosphoric '
                                        'triamides, P(=O)(NR2)3, since '
                                        'replacement of one or two OH groups '
                                        'produces phosphoramidic acids: '
                                        'P(=O)(OH)(NR2)2 , P(=O)(OH)2(NR2).',
                          'parents': ['CHEBI:33256']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0.09009009009009009 is too low.\n'
               "True positives: [('OS(=O)CCNC(=N)NP(O)(O)=O', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('N(CCCl)(CCCl)P1(=O)N(CCCl)CCCO1', 'Phosphoramide with 2 N "
               "atoms and 0 OH groups'), ('OC(=O)CCOP(=O)(NCCCl)NCCCl', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('CN(C)P(F)(=O)N(C)C', 'Phosphoramide with 2 N atoms and 0 OH "
               "groups'), ('NC(=N)NCCCCNP(O)(O)=O', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups')]\n"
               'False positives: '
               "[('C1(=CN(C(NC1=O)=O)C/C=C/CP(N[C@H](C(OCC2=CC=CC=C2)=O)C)(=O)OC3=CC=CC=C3)Cl', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('Nc1nc2n(cnc2c(=O)[nH]1)[C@@H]1O[C@H](COP(O)(=O)OP(O)(=O)NP(O)(O)=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[C@H]1(O)[C@H](N2C(N=C(C=C2)N)=O)O[C@H](COP(OP(=O)(N)[O-])(=O)[O-])[C@H]1OP(=O)([O-])[O-]', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('COP([O-])(=O)OCCNC(=[NH2+])NP([O-])([O-])=O', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('NCCCOP(=O)(NC(=O)[C@@H](N)CC([O-])=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(OC1=CC=CC=C1)(OC2=CC=CC=C2)(=O)/N=C(\\\\OC)/N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(OC[C@H]1O[C@@H](N2C(=O)NC=3C2=NC=NC3N)[C@@H]([C@@H]1O)O)(OC)N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('N[C@@H](CCCNC(=N)NP(O)(O)=O)C(O)=O', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('P(=O)(N1CC1)(N2CC2)NC(OCC3=CC=CC=C3)=O', 'Phosphoric "
               "triamide P(=O)(NR2)3'), "
               "('FC(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)CCOP(=O)(N1CCOCC1)N1CCOCC1', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('P(O)(O)(=O)N\\\\C(=N\\\\CCCCN)\\\\N', 'Phosphoramide with 1 "
               "N atoms and 0 OH groups'), "
               "('NC1=NC=NC2=C1N=CN2[C@@H]3O[C@H](COP(=O)(N)O)[C@@H](O)[C@H]3O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(O)(O)(=O)N\\\\C(=N\\\\CCCC(N)C(O)=O)\\\\N', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('CN1C=C(C=C1P(=S)(N2CCOCC2)N3CCOCC3)P(=O)(N4CCOCC4)N5CCOCC5', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('NC1=NC=NC2=C1N=CN2[C@@H]3O[C@H](COP(=O)(N4CCOCC4)O)[C@@H](O)[C@H]3O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(O)(O)N[C@H]1[C@H](O[C@H](CO)[C@H]([C@@H]1O)O)O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[NH3+][C@@H](CCCNC(=[NH2+])NP([O-])([O-])=O)C([O-])=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CN1C(C=C(CC1=NP(=O)(OC2=CC=CC=C2)OC3=CC=CC=C3)C4=CC=C(C=C4)CO)C5CC5', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[NH3+][C@@H](COP([O-])(=O)OCCNC(=[NH2+])NP([O-])([O-])=O)C([O-])=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP(N)([O-])=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('N1(P(OC[C@H]2O[C@@H](N3C=4N=C(NC(=O)C4N=C3)N)[C@@H]([C@@H]2O)O)(=O)[O-])C=C(N=C1)C[C@@H](C(*)=O)N*', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('NP(=O)(OCCC(O)=O)N(CCCl)CCCl', 'Phosphoramide with 2 N atoms "
               "and 0 OH groups'), ('CCOP(=O)(SC(C)CC)N1CCSC1=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[C@H]1(O)[C@H](N2C(N=C(C=C2)N)=O)O[C@H](COP(OP(=O)(N)[O-])(=O)[O-])[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[O-]P([O-])(=O)NC(=[NH2+])NCCS([O-])(=O)=O', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('CN(CC([O-])=O)C(=[NH2+])NP([O-])([O-])=O', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('CC(O)=O.[H][C@]12SCC(Sc3nc(cs3)-c3cc[n+](C)cc3)=C(N1C(=O)[C@H]2NC(=O)C(=N/OCC)\\\\c1nsc(NP(O)(O)=O)n1)C([O-])=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(O)(N(NC(=O)[C@@H](N)CCCN=C(N)N)C)C(O)C(=O)OC', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[O-]P([O-])(=O)n1cnc(C[C@H](N-*)C(-*)=O)c1', 'Phosphoramide "
               "with 1 N atoms and 0 OH groups'), "
               "('P(O)(O)(=O)/N=C\\\\1/N(CC(=O)N1)C', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('[O-]S(=O)CCNC(=[NH2+])NP([O-])([O-])=O', 'Phosphoramide with "
               "1 N atoms and 0 OH groups'), "
               "('CCOP(=O)(OCC)\\\\N=C1/SCC(C)S1', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('NCCCOP(=O)(NC(=O)[C@@H](N)CC(O)=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[C@@H]1(O[C@H]([C@@H]([C@@H]1O)O)N2C=3N=C(NC(=O)C3[N+](=C2)C)N)COP([O-])(=O)N4C=C(N=C4)C[C@@H](C(=O)*)N*', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('S(=O)(=NP(=O)(O)O)(CCC(N)C(=O)NC(C(=O)NC(C(=O)O)C)C)C', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('O.ClCCN(CCCl)P1(=O)NCCCO1', 'Phosphoramide with 2 N atoms "
               "and 0 OH groups'), "
               "('NCCCCCCCCNP(O)(=O)OP(O)(=O)OP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c1nc(N)[nH]c2=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CN1C(=CN=C1[N+](=O)[O-])COP(=O)(NCCBr)NCCBr', 'Phosphoramide "
               "with 2 N atoms and 0 OH groups'), "
               "('FC(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)CCCCCCCCCCCOP(=O)(N1CCOCC1)N1CCOCC1', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('COP(O)(=O)OCCNC(=N)NP(O)(O)=O', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), ('P(=O)(N=C(N(CC)CC)C)(OCC)F', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(OC[C@H]1O[C@@H](N2C3=C(C(=NC=N3)N)N=C2)[C@@H]([C@@H]1O)O)(NC4=C(C=CC=C4)C(O)=O)(=O)O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(N=C(N(CC)CC)C)(C)F', 'Phosphoramide with 1 N atoms and "
               "0 OH groups'), "
               "('CN(C)c1ncnc2n(cnc12)[C@@H]1O[C@H](COP(=O)(NCCCC[C@H](NC(C)=O)C(O)=O)N2CCOCC2)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('N1(P(=O)([O-])OC[C@H]2O[C@@H](N3C=4N=CN=C(N)C4N=C3)[C@@H]([C@@H]2OP(OC[C@H]5O[C@@H](N6C=7N=CN=C(N)C7N=C6)[C@@H]([C@@H]5OP(OC[C@H]8O[C@@H](N9C(NC(=O)C(=C9)C)=O)[C@@H]([C@@H]8OP(OC[C@H]%10O[C@@H](N%11C=%12N=CN=C(N)C%12N=C%11)[C@@H]([C@@H]%10*)O)(=O)[O-])O)(=O)[O-])O)(=O)[O-])O)C=C(N=C1)C[C@@H](C(=O)*)N*', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CCOP(=O)(OCC)N=C1SCCS1', 'Phosphoramide with 1 N atoms and 0 "
               "OH groups'), "
               "('FC(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)CCCCCCCCCCCOP(=O)(N1CCOCC1)N1CCOCC1', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('P(=O)(ONCCCCCCCCCCCC(C)C)(O)N(P(=O)(OC1C(OCC2OC(N3C(=O)N=C(N)C=C3)C(C2O)O)=C(CO)C(C1O)O)O)CC(=O)N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('ClCCN(CCCl)P1(=O)NC(=O)CCO1', 'Phosphoramide with 2 N atoms "
               "and 0 OH groups'), ('CCCSP(=O)(OCC)N1CCN(CC)\\\\C1=N/C#N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('N[C@@H](Cc1cncn1P(O)(O)=O)C(O)=O', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('COC(=O)[C@H](CCCCNP([O-])(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c1nc(N)[nH]c2=O)NC(C)=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(N=C(N(CC)CC)C)(OC)F', 'Phosphoramide with 1 N atoms "
               "and 0 OH groups'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP([O-])(=O)Nc2ccccc2C([O-])=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('NC(=[NH2+])NCCCCNP([O-])([O-])=O', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('OC(=O)CCCC(=O)Nc1ccc(CP(O)(=O)Nc2ccc(cc2)[N+]([O-])=O)cc1', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('S(=O)(=O)(O)NP(=O)(N1C(=O)[C@H](N)CCC1)N', 'Phosphoric "
               "triamide P(=O)(NR2)3'), "
               "('[C@@H]1(O[C@H]([C@@H]([C@@H]1O*)O)*)COP([O-])(=O)N2C=C(N=C2)C[C@@H](C(=O)*)N*', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('S(=O)(=O)(O)N[P@](=O)(N1C(=O)[C@@H](N)CCC1)N', 'Phosphoric "
               "triamide P(=O)(NR2)3'), "
               "('[C@H]1(O)[C@H](N2C(N=C(C=C2)N)=O)O[C@H](COP(OP(=O)(NC(CC[C@@H](C(=O)[O-])[NH3+])=O)[O-])(=O)[O-])[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('NP1(=O)OCCCN1CCCl', 'Phosphoramide with 2 N atoms and 0 OH "
               "groups'), ('[P@](=O)(OC1=CC=C(/C=C/C(=O)OC)C=C1)(OC)N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(OC[C@H]1O[C@@H](N2C(=O)NC=3C2=NC=NC3N)[C@@H]([C@@H]1O)O)(O)NC(=O)[C@H]4NCCC4', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('C1(=O)NC(=NC2=C1N=CN2[C@@H]3O[C@H](COP(=O)(NCCCC[C@@H](C(*)=O)N*)[O-])[C@@H](O)[C@H]3O)N', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('C(C(=O)NP([O-])(=O)[O-])C[C@@H](C(=O)[O-])[NH3+]', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(OC)(O)N(NC(=O)[C@@H](N)CC(C)C)C', 'Phosphoramide with "
               "1 N atoms and 0 OH groups'), "
               "('[O-]P([O-])(=O)NC(=[NH2+])NCCC[C@H](N-*)C(-*)=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CNC[C@H](O)[C@@H](O)[C@H](O)[C@H](O)CO.CNC[C@H](O)[C@@H](O)[C@H](O)[C@H](O)CO.C[C@@H](O[C@H]1OCCN(Cc2nn(c(=O)[nH]2)P(O)(O)=O)[C@H]1c1ccc(F)cc1)c1cc(cc(c1)C(F)(F)F)C(F)(F)F', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('C(C(=O)NP(O)(=O)O)C[C@@H](C(=O)O)N', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), ('NP(O)(O)=O', 'Phosphoramide with 1 "
               "N atoms and 0 OH groups'), "
               "('P(N(CCCl)CCCl)(N(CCCl)CCCl)(OCCS(C[C@@H](C(N[C@@H](C(O)=O)C1=CC=CC=C1)=O)NC(CC[C@@H](C(O)=O)N)=O)(=O)=O)=O', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('C1[C@H]([C@@H]1C(=O)NC2=CC3=CC=CC=C3C=C2)[C@H](C4=CC=CC=C4)NP(=O)(C5=CC=CC=C5)C6=CC=CC=C6', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CN(CC(O)=O)C(=N)NP(O)(O)=O', 'Phosphoramide with 1 N atoms "
               "and 0 OH groups'), "
               "('CC1=C(C=C(C=C1)NP(=O)(OC2=CC=CC=C2F)OC3=CC=CC=C3F)C', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(=O)(OC[C@H]1O[C@@H](N2C(=O)NC=3C2=NC=NC3N)[C@@H]([C@@H]1O)O)(OC)NC(=O)[C@H]4NCCC4', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CN(C)P(=O)(N(C)C)n1nc(nc1N)-c1ccccc1', 'Phosphoric triamide "
               "P(=O)(NR2)3'), "
               "('P(=O)(OCC1OC(OC2C(O)C(O)C(OCC(OC(=O)CCCCCCC/C=C\\\\C/C=C\\\\CCCCC)COC(=O)CCCCCCCCCCC/C=C\\\\CCCCCCCC)OC2CO)C(O)C(C1O)O)(O)N[C@H](C(=O)N[C@H](C(=O)O)CC3=CC=CC=C3)CC4=CC=CC=C4', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('P(Cl)(*)(N)=O', 'Phosphoramide with 1 N atoms and 0 OH "
               "groups'), ('S(=O)(=O)(C1=C(C=C(OP(OCC)(=O)NC(C)C)C=C1)C)C', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('NP(=O)(OCCC=O)N(CCCl)CCCl', 'Phosphoramide with 2 N atoms "
               "and 0 OH groups'), "
               "('Nc1nc2n(cnc2c(=O)[nH]1)[C@@H]1O[C@H](COP([O-])(=O)OP([O-])(=O)NP([O-])([O-])=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[O-]C(=O)CNC(=[NH2+])NP([O-])([O-])=O', 'Phosphoramide with "
               "1 N atoms and 0 OH groups'), "
               "('CSCC[C@H](NC=O)C(=O)N[C@@H](CCCNC(N)=N)C(=O)N[C@@H]([C@@H](C)O)C(=O)NCC(=O)N[C@@H](CC(N)=O)C(=O)N[C@@H](C)C(=O)N[C@@H](CC(O)=O)C(=O)NP(=O)(OCCCN)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c(N)ncnc12', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CNP(=O)(OC)Oc1ccc(cc1Cl)C(C)(C)C', 'Phosphoramide with 1 N "
               "atoms and 0 OH groups'), "
               "('P(=O)(O[C@H]1O[C@H]([C@@H](O)[C@H]([C@H]1O)O)C)(O)N[C@H](C(=O)N[C@H](C(=O)O)CC=2C3=C(C=CC=C3)NC2)CC(C)C', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('COC(=O)[C@H](CCCCNP(O)(=O)OC[C@H]1O[C@H]([C@H](O)[C@@H]1O)n1cnc2c1nc(N)[nH]c2=O)NC(C)=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP([O-])(=O)N[C@@H](Cc2ccccc2)C([O-])=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('C[C@@H](O[C@H]1OCCN(Cc2nn(c(=O)[nH]2)P([O-])([O-])=O)[C@H]1c1ccc(F)cc1)c1cc(cc(c1)C(F)(F)F)C(F)(F)F', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('NP(=O)(OCCCO)N(CCCl)CCCl', 'Phosphoramide with 2 N atoms and "
               "0 OH groups'), "
               "('O.CC(O)=O.[H][C@]12SCC(Sc3nc(cs3)-c3cc[n+](C)cc3)=C(N1C(=O)[C@H]2NC(=O)C(=N/OCC)\\\\c1nsc(NP(O)(O)=O)n1)C([O-])=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('[C@H](NC([C@@H](NC([C@H](CCCNP(NS(O)(=O)=O)(=O)N)N)=O)C)=O)(CCCNC(N)=N)C(O)=O', "
               "'Phosphoric triamide P(=O)(NR2)3'), "
               "('NC(CCC(=O)NC(CSC1CCOP(=O)(N1)N(CCCl)CCCl)C(=O)NCC(O)=O)C(O)=O', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('Nc1ncnc2n(cnc12)[C@@H]1O[C@H](COP(O)(=O)N[C@@H](Cc2ccccc2)C(O)=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('N(CCCl)(CCCl)[P@@]1(=O)N[C@H](OO)CCO1', 'Phosphoramide with "
               "2 N atoms and 0 OH groups'), ('NP([O-])([O-])=O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('FC(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)C(F)(F)CCOP(=O)(N1CCOCC1)N1CCOCC1', "
               "'Phosphoramide with 2 N atoms and 0 OH groups'), "
               "('CCOP(=O)(OCC)N=C1SCS1', 'Phosphoramide with 1 N atoms and 0 "
               "OH groups'), "
               "('[H+].[Na+].[Na+].[Na+].Nc1nc2n(cnc2c(=O)[nH]1)[C@@H]1O[C@H](COP([O-])(=O)OP([O-])(=O)NP([O-])([O-])=O)[C@@H](O)[C@H]1O', "
               "'Phosphoramide with 1 N atoms and 0 OH groups'), "
               "('CN(C1=CC=CC=C1)P(=O)(N(C)C2=CC=CC=C2)N(C)C3=CC=CC=C3', "
               "'Phosphoric triamide P(=O)(NR2)3'), "
               "('P(OC1OC(C(O)C1O)C(O)CO)(O)(=O)NC=2N=CN=C3N(C4OC(CC4O)COP(O)(=O)NC(=O)C(O)C(O)C(C)C)C=NC32', "
               "'Phosphoramide with 1 N atoms and 0 OH groups')]\n"
               'False negatives: '
               "[('S1C(=NC(=C1)C2=NC(=C(O)C(=C2OC)OC)C(=O)N)[C@@H](O)CCCC', "
               "'No phosphorus atoms found')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 5,
    'num_false_positives': 91,
    'num_true_negatives': 183780,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.052083333333333336,
    'recall': 0.8333333333333334,
    'f1': 0.09803921568627451,
    'accuracy': 0.9994996655372885}


"""
Classifies: CHEBI:18000 aralkylamine
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_aralkylamine(smiles: str):
    """
    Determines if a molecule is an aralkylamine (alkylamine with aromatic substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an aralkylamine, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Look for amine groups (-NH2, -NHR, -NR2) not part of amides/sulfonamides/aromatic rings
    pattern_amine = Chem.MolFromSmarts('[NX3;H2,H1,H0;!$(NC=O);!$(NS=O);!$(N=*);!$(n)]')
    if pattern_amine is None:
        return None, "Invalid amine SMARTS pattern"
    amine_matches = mol.GetSubstructMatches(pattern_amine)
    
    if not amine_matches:
        return False, "No aliphatic amine groups found"

    # Look for aromatic rings
    aromatic_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetIsAromatic()]
    if not aromatic_atoms:
        return False, "No aromatic rings found"
    
    # For each amine, check if it's connected to an aromatic ring through a carbon chain
    for amine_match in amine_matches:
        amine_idx = amine_match[0]
        amine_atom = mol.GetAtomWithIdx(amine_idx)
        
        # Get neighboring carbon atoms of the amine
        carbon_neighbors = [neighbor for neighbor in amine_atom.GetNeighbors() 
                          if neighbor.GetSymbol() == 'C' and not neighbor.GetIsAromatic()]
        
        for carbon in carbon_neighbors:
            # Check if this carbon is part of a path to an aromatic ring
            visited = set()
            stack = [(carbon, [carbon.GetIdx()])]
            
            while stack:
                current_atom, path = stack.pop()
                
                if current_atom.GetIdx() in visited:
                    continue
                    
                visited.add(current_atom.GetIdx())
                
                if current_atom.GetIsAromatic():
                    chain_length = len(path) - 1  # Exclude the aromatic atom
                    ring_type = "aromatic ring"
                    if current_atom.GetSymbol() == 'C':
                        ring_type = "benzene ring"
                    elif current_atom.GetSymbol() == 'N':
                        ring_type = "pyridine ring"
                    elif current_atom.GetSymbol() == 'O':
                        ring_type = "furan ring"
                    elif current_atom.GetSymbol() == 'S':
                        ring_type = "thiophene ring"
                    return True, f"Found aralkylamine with {chain_length}-carbon chain connecting {ring_type} to amine"
                
                for neighbor in current_atom.GetNeighbors():
                    if (neighbor.GetSymbol() == 'C' and neighbor.GetIdx() not in path) or neighbor.GetIsAromatic():
                        new_path = path + [neighbor.GetIdx()]
                        stack.append((neighbor, new_path))
                        
    return False, "No alkyl chain connecting aromatic ring to amine found"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:18000',
                          'name': 'aralkylamine',
                          'definition': 'An alkylamine in which the alkyl '
                                        'group is substituted by an aromatic '
                                        'group.',
                          'parents': ['CHEBI:22331', 'CHEBI:64365']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               "False negatives: [('O1C(C=2C(CNCC1)=CC=CC2)C3=CC=CC=C3', "
               "'Invalid aralkyl SMARTS pattern'), "
               "('C1NC(=NCN1CC2=CC=CC=C2)NC#N', 'Invalid aralkyl SMARTS "
               "pattern'), ('C1=CC=C(C=C1)CNC2=NC(=O)C(=CC3=CC=C(C=C3)O)S2', "
               "'Invalid aralkyl SMARTS pattern'), "
               "('CC1=C(C(=NC1=C2CSC(=NN2)NCC3=CC=CC=C3)C)C(=O)C', 'Invalid "
               "aralkyl SMARTS pattern'), ('O1C(C2NCCCC2)=CC=C1C', 'No "
               "aromatic rings found'), "
               "('C1=CC=C(C=C1)CNC2=NC(=O)C(=CC3=CC=CC=C3)S2', 'Invalid "
               "aralkyl SMARTS pattern'), "
               "('O(C(=O)[C@@H]([C@@]1(NCCCC1)[H])C2=CC=CC=C2)CC', 'Invalid "
               "aralkyl SMARTS pattern'), "
               "('CC1=CC=C(O1)C(=O)CCNC2=CC(=CC=C2)Br', 'Invalid aralkyl "
               "SMARTS pattern'), "
               "('C[C@H]1CN(C(=O)C2=CC=CC=C2C3=CC=CC=C3CO[C@@H]1CN(C)CC4=CN=CC=C4)[C@@H](C)CO', "
               "'Invalid aralkyl SMARTS pattern'), "
               "('CCNC(=O)C[C@H]1CC[C@@H]2[C@@H](O1)COC[C@@H](CN2CC3=CC=NC=C3)O', "
               "'Invalid aralkyl SMARTS pattern'), "
               "('O[C@H]1[C@H](NCC(C)C)[C@H](C[C@H]1O)C=2C=CC=NC2', 'Invalid "
               "aralkyl SMARTS pattern'), ('S1C(CNC)=CC=C1C2=CC=NC=C2', "
               "'Invalid aralkyl SMARTS pattern'), "
               "('C1=COC(=C1)C2=CC(=NC(=N2)NCC3=CC=C(C=C3)Cl)C(F)(F)F', "
               "'Invalid aralkyl SMARTS pattern')]",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 13,
    'num_false_positives': 100,
    'num_true_negatives': 1019,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.11504424778761062,
    'recall': 1.0,
    'f1': 0.20634920634920637,
    'accuracy': 0.911660777385159}


"""
Classifies: CHEBI:23437 cyanohydrin
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_cyanohydrin(smiles: str):
    """
    Determines if a molecule is a cyanohydrin (alpha-hydroxynitrile).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cyanohydrin, False otherwise
        str: Reason for classification
    """
    # Parse SMILES
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"
        
    # Find carbons with both OH and CN groups
    cyanohydrin_carbons = []
    for atom in mol.GetAtoms():
        if atom.GetSymbol() != 'C':
            continue
            
        # Check for CN group
        has_cn = False
        has_oh = False
        
        for neighbor in atom.GetNeighbors():
            if neighbor.GetSymbol() == 'N':
                # Check if N is part of CN triple bond
                if neighbor.GetDegree() == 1 and len([b for b in neighbor.GetBonds() if b.GetBondType() == Chem.BondType.TRIPLE]) == 1:
                    has_cn = True
            elif neighbor.GetSymbol() == 'O':
                # Check if O is part of OH group
                if neighbor.GetDegree() == 1:
                    has_oh = True
                    
        if has_cn and has_oh:
            cyanohydrin_carbons.append(atom.GetIdx())
            
    if not cyanohydrin_carbons:
        return False, "No carbon with both OH and CN groups found"
        
    # For each potential cyanohydrin carbon, check if it was derived from aldehyde/ketone
    for c_idx in cyanohydrin_carbons:
        c_atom = mol.GetAtomWithIdx(c_idx)
        
        # Count number of carbon neighbors (excluding CN)
        carbon_neighbors = 0
        for neighbor in c_atom.GetNeighbors():
            if neighbor.GetSymbol() == 'C':
                # Don't count the CN carbon
                if not any(n.GetSymbol() == 'N' and n.GetDegree() == 1 for n in neighbor.GetNeighbors()):
                    carbon_neighbors += 1
                    
        # Must have 0-2 carbon neighbors (excluding CN)
        # 0 = formaldehyde-derived
        # 1 = aldehyde-derived  
        # 2 = ketone-derived
        if carbon_neighbors <= 2:
            if carbon_neighbors == 0:
                return True, "Formaldehyde-derived cyanohydrin"
            elif carbon_neighbors == 1:
                return True, "Aldehyde-derived cyanohydrin"
            else:
                return True, "Ketone-derived cyanohydrin"
                
    return False, "Structure does not match cyanohydrin pattern"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23437',
                          'name': 'cyanohydrin',
                          'definition': 'An alpha-hydroxynitrile resulting '
                                        'from the formal addition of hydrogen '
                                        'cyanide to the C=O bond of an '
                                        'aldehyde or ketone.',
                          'parents': ['CHEBI:22455']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 3,
    'num_true_negatives': 183895,
    'num_false_negatives': 3,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999673737500068}


"""
Classifies: CHEBI:17634 D-glucose
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_D_glucose(smiles: str):
    """
    Determines if a molecule is a D-glucose, i.e., a glucose with D-configuration.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a D-glucose, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check if the molecule contains 6 carbon atoms and 1 oxygen atom
    num_carbon_atoms = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    num_oxygen_atoms = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'O')
    if num_carbon_atoms != 6 or num_oxygen_atoms != 1:
        return False, "Molecule does not have the correct number of carbon (6) and oxygen (1) atoms"

    # Get the list of SMILES for each stereoisomer
    isomers = AllChem.EnumerateStereoisomers(mol)
    isomers_smiles = [Chem.MolToSmiles(isomer) for isomer in isomers]

    # Check if any of the stereoisomers match the D-glucose configuration
    d_glucose_smiles = [
        'O[C@H]1[C@@H]([C@@H]([C@H]([C@@H](O1)O)O)O)O',  # D-glucopyranose
        'O1[C@@H]([C@H](O)[C@@H](O)C1O)[C@H](O)CO'       # D-glucofuranose
    ]
    if any(smiles in isomers_smiles for smiles in d_glucose_smiles):
        return True, "Molecule is a D-glucose"
    else:
        return False, "Molecule is not a D-glucose"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17634',
                          'name': 'D-glucose',
                          'definition': 'A glucose with D-configuration.',
                          'parents': ['CHEBI:17234', 'CHEBI:17608']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183925,
    'num_false_negatives': 1,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999945630307842}


"""
Classifies: CHEBI:131657 cholane derivative
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem.AllChem import GetMorganFingerprintAsBitVect
from rdkit.Chem import rdMolDescriptors

def is_cholane_derivative(smiles: str):
    """
    Determines if a molecule is a cholane derivative based on steroid skeleton.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a cholane derivative, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for basic molecular properties consistent with cholane derivatives
    num_carbons = sum(1 for atom in mol.GetAtoms() if atom.GetSymbol() == 'C')
    if num_carbons < 24:  # Cholane skeleton has 24 carbons
        return False, "Too few carbons for cholane skeleton"

    # Count number of rings
    ring_info = mol.GetRingInfo()
    if len(ring_info.AtomRings()) < 4:
        return False, "Missing required ring systems"

    # Check for steroid core - should have 4 fused rings
    ring_counts = rdMolDescriptors.CalcNumRings(mol)
    if ring_counts < 4:
        return False, "Does not contain required 4-ring steroid core"

    # Check for characteristic 6-6-6-5 ring pattern of steroids
    rings = ring_info.AtomRings()
    six_membered = sum(1 for ring in rings if len(ring) == 6)
    five_membered = sum(1 for ring in rings if len(ring) == 5)
    
    if six_membered < 3 or five_membered < 1:
        return False, "Does not match characteristic steroid ring pattern (6-6-6-5)"

    # Look for characteristic methyl groups at C-18 and C-19 positions
    # This is a simplified check for branching patterns
    branching_points = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C' and len([n for n in atom.GetNeighbors()]) == 4:
            branching_points += 1

    if branching_points < 2:
        return False, "Missing characteristic methyl branching points"

    # If all checks pass, it's likely a cholane derivative
    return True, "Contains cholane skeleton with characteristic steroid core structure"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:131657',
                          'name': 'cholane derivative',
                          'definition': 'Any steroid (or derivative) based on '
                                        'a cholane skeleton.',
                          'parents': ['CHEBI:35341']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 100,
    'num_true_negatives': 2128,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0196078431372549,
    'recall': 1.0,
    'f1': 0.038461538461538464,
    'accuracy': 0.9551569506726457}


"""
Classifies: CHEBI:23073 glycopentaosylceramide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import rdMolDescriptors

def is_glycopentaosylceramide(smiles: str):
    """
    Determines if a molecule is a glycopentaosylceramide - an oligoglycosylceramide with 5 sugar units
    attached to ceramide.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a glycopentaosylceramide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for ceramide core
    ceramide_pattern = Chem.MolFromSmarts("[CH2]-[CH]-[CH]-N-C(=O)-*") 
    if not mol.HasSubstructMatch(ceramide_pattern):
        return False, "No ceramide core found"

    # Count sugar units (pyranose rings)
    sugar_pattern = Chem.MolFromSmarts("[C]1[C][C][C][C]([C]1)O")
    sugar_matches = mol.GetSubstructMatches(sugar_pattern)
    
    if len(sugar_matches) != 5:
        return False, f"Found {len(sugar_matches)} sugar units instead of required 5"

    # Check for glycosidic linkages between sugars
    glycosidic_pattern = Chem.MolFromSmarts("[C]-O-[C]")
    glycosidic_matches = mol.GetSubstructMatches(glycosidic_pattern)
    
    if len(glycosidic_matches) < 5:
        return False, "Insufficient glycosidic linkages between sugar units"

    # Check for hydroxyl groups characteristic of sugars
    hydroxyl_count = len(mol.GetSubstructMatches(Chem.MolFromSmarts("O[CH]")))
    if hydroxyl_count < 15: # Each sugar typically has 3-4 OH groups
        return False, "Insufficient hydroxyl groups for pentasaccharide structure"

    # If all checks pass, this is likely a glycopentaosylceramide
    return True, "Contains ceramide core with 5 sugar units connected by glycosidic bonds"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23073',
                          'name': 'glycopentaosylceramide',
                          'definition': 'An oligoglycosylceramide consisting '
                                        'of a glycopentaosyl moiety attached '
                                        'to the ceramide oxygen with an '
                                        'unspecified N-acyl substituent '
                                        'attached to the ceramide nitrogen.',
                          'parents': ['CHEBI:36520']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183838,
    'num_false_negatives': 9,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9999510462504148}


"""
Classifies: CHEBI:22315 alkaloid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_alkaloid(smiles: str):
    """
    Determines if a molecule is an alkaloid based on the definition:
    'Any of the naturally occurring, basic nitrogen compounds (mostly heterocyclic) 
    occurring mostly in the plant kingdom, but also found in bacteria, fungi, and animals.'
    
    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alkaloid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for presence of nitrogen
    if not any(atom.GetAtomicNum() == 7 for atom in mol.GetAtoms()):
        return False, "No nitrogen atoms present"
    
    # Count number of nitrogen atoms
    n_count = sum(1 for atom in mol.GetAtoms() if atom.GetAtomicNum() == 7)
    
    # Check for heterocycles containing nitrogen
    rings = mol.GetRingInfo()
    n_in_ring = False
    for ring in rings.AtomRings():
        ring_atoms = [mol.GetAtomWithIdx(i) for i in ring]
        if any(atom.GetAtomicNum() == 7 for atom in ring_atoms):
            n_in_ring = True
            break
    
    # Exclude peptides/proteins (high nitrogen content and peptide bonds)
    peptide_like = False
    for bond in mol.GetBonds():
        if (bond.GetBondType() == Chem.BondType.SINGLE and
            bond.GetBeginAtom().GetAtomicNum() == 6 and
            bond.GetEndAtom().GetAtomicNum() == 7 and
            any(n.GetAtomicNum() == 8 for n in bond.GetBeginAtom().GetNeighbors()) and
            any(n.GetAtomicNum() == 8 for n in bond.GetEndAtom().GetNeighbors())):
            peptide_like = True
            break

    # Exclude nucleotides/nucleic acids (presence of phosphate groups)
    has_phosphate = any(atom.GetAtomicNum() == 15 for atom in mol.GetAtoms())
    
    # Check for basic nitrogen (sp3 nitrogen with lone pair)
    basic_n = False
    for atom in mol.GetAtoms():
        if (atom.GetAtomicNum() == 7 and 
            atom.GetHybridization() == Chem.HybridizationType.SP3 and 
            not atom.GetFormalCharge()):
            basic_n = True
            break

    if peptide_like:
        return False, "Structure appears to be peptide-like"
    
    if has_phosphate:
        return False, "Contains phosphate group, likely nucleotide/nucleic acid"

    if not n_in_ring and basic_n:
        return False, "Contains only exocyclic amine, likely biogenic amine rather than alkaloid"
        
    if n_in_ring and basic_n:
        return True, "Contains basic nitrogen in heterocyclic ring system"
        
    if n_in_ring:
        return True, "Contains nitrogen heterocycle"
        
    return False, "Does not meet alkaloid criteria"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:22315',
                          'name': 'alkaloid',
                          'definition': 'Any of the naturally occurring, basic '
                                        'nitrogen compounds (mostly '
                                        'heterocyclic) occurring mostly in the '
                                        'plant kingdom, but also found in '
                                        'bacteria, fungi, and animals. By '
                                        'extension, certain neutral compounds '
                                        'biogenetically related to basic '
                                        'alkaloids are also classed as '
                                        'alkaloids. Amino acids, peptides, '
                                        'proteins, nucleotides, nucleic acids, '
                                        'amino sugars and antibiotics are not '
                                        'normally regarded as alkaloids. '
                                        'Compounds in which the nitrogen is  '
                                        'exocyclic (dopamine, mescaline, '
                                        'serotonin, etc.) are usually classed '
                                        'as amines rather than alkaloids.',
                          'parents': ['CHEBI:35352']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 267,
    'num_false_positives': 100,
    'num_true_negatives': 155,
    'num_false_negatives': 6,
    'num_negatives': None,
    'precision': 0.7275204359673024,
    'recall': 0.978021978021978,
    'f1': 0.8343749999999999,
    'accuracy': 0.7992424242424242}


"""
Classifies: CHEBI:25681 omega-3 fatty acid
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_omega_3_fatty_acid(smiles: str):
    """
    Determines if a molecule is an omega-3 fatty acid.
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is an omega-3 fatty acid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for carboxylic acid group
    carboxylic_pattern = Chem.MolFromSmarts('C(=O)[OH]')
    if len(mol.GetSubstructMatches(carboxylic_pattern)) == 0:
        return False, "No carboxylic acid group found"

    # Find the carbon chain
    # First find the carboxylic carbon
    matches = mol.GetSubstructMatches(carboxylic_pattern)
    if not matches:
        return False, "Could not identify carboxylic carbon"
    carboxylic_carbon = matches[0][0]
    
    # Get all carbons in longest chain from carboxylic end
    def get_carbon_chain(atom_idx, visited=None):
        if visited is None:
            visited = set()
            
        atom = mol.GetAtomWithIdx(atom_idx)
        if atom.GetSymbol() != 'C':
            return []
            
        visited.add(atom_idx)
        current_chain = [atom_idx]
        
        # Get neighboring carbons
        neighbors = [n for n in atom.GetNeighbors() 
                    if n.GetSymbol() == 'C' and n.GetIdx() not in visited]
        
        # Continue along longest chain
        max_subchain = []
        for n in neighbors:
            subchain = get_carbon_chain(n.GetIdx(), visited.copy())
            if len(subchain) > len(max_subchain):
                max_subchain = subchain
                
        return current_chain + max_subchain
    
    chain = get_carbon_chain(carboxylic_carbon)
    
    if len(chain) < 4:  # Need at least 4 carbons
        return False, "Carbon chain too short"
        
    # Check for double bond at omega-3 position (3rd from methyl end)
    methyl_end = chain[-1]
    omega3_carbon = chain[-3]
    
    # Get bond between omega-3 carbon and next carbon
    for neighbor in mol.GetAtomWithIdx(omega3_carbon).GetNeighbors():
        if neighbor.GetIdx() in chain:
            bond = mol.GetBondBetweenAtoms(omega3_carbon, neighbor.GetIdx())
            if bond.GetBondType() == Chem.rdchem.BondType.DOUBLE:
                return True, "Found double bond at omega-3 position"
                
    return False, "No double bond at omega-3 position"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:25681',
                          'name': 'omega-3 fatty acid',
                          'definition': 'A family of polyunsaturated fatty '
                                        'acids that have in common a final '
                                        'carbon-carbon double bond in the '
                                        'omega-3 position, i.e., the third '
                                        'bond from the methyl end of the fatty '
                                        'acid.',
                          'parents': ['CHEBI:26208', 'CHEBI:59549']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': "Attempt failed: 'Mol' object has no attribute "
               "'HasSubstructMatches'",
    'attempt': 3,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 3,
    'num_false_positives': 100,
    'num_true_negatives': 8422,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.02912621359223301,
    'recall': 1.0,
    'f1': 0.05660377358490566,
    'accuracy': 0.9882697947214076}


"""
Classifies: CHEBI:166828 saccharolipid
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_saccharolipid(smiles: str):
    """
    Determines if a molecule is a saccharolipid (lipid containing a carbohydrate moiety).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a saccharolipid, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check for presence of carbohydrate moiety
    # Look for cyclic structures with multiple OH groups and O in ring
    rings = mol.GetRingInfo()
    if not rings.NumRings():
        return False, "No rings found - carbohydrate moiety requires cyclic structure"

    # Find rings that could be carbohydrates (5-6 membered with oxygen)
    sugar_rings = []
    for ring in rings.AtomRings():
        if len(ring) in [5,6]:
            atoms = [mol.GetAtomWithIdx(i) for i in ring]
            if any(atom.GetSymbol() == 'O' for atom in atoms):
                sugar_rings.append(ring)
                
    if not sugar_rings:
        return False, "No potential carbohydrate rings found"

    # Check for lipid characteristics - long carbon chain and/or fatty acid
    has_long_chain = False
    has_fatty_acid = False
    
    # Look for carbonyl groups (C=O) connected to O 
    pattern = Chem.MolFromSmarts('[CX3](=[OX1])[OX2]')
    if mol.HasSubstructMatch(pattern):
        has_fatty_acid = True
        
    # Look for alkyl chains of length >= 6
    pattern = Chem.MolFromSmarts('CCCCCC')
    if mol.HasSubstructMatch(pattern):
        has_long_chain = True

    if not (has_long_chain or has_fatty_acid):
        return False, "No lipid characteristics found (missing long carbon chain or fatty acid)"

    return True, "Contains both carbohydrate moiety and lipid characteristics"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:166828',
                          'name': 'saccharolipid',
                          'definition': 'Lipids that contain a carbohydrate '
                                        'moiety.',
                          'parents': ['CHEBI:18059']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': '\n'
               'Attempt failed: F1 score of 0 is too low.\n'
               'True positives: []\n'
               'False positives: []\n'
               'False negatives: '
               "[('[C@H]1(OP(=O)(O)O)[C@H](O*)[C@@H](N*)[C@@H](O[C@@H]1CO)OC[C@@H]2[C@H]([C@@H]([C@H]([C@H](O2)OP(=O)(O)O)N*)O*)O', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@H]1[C@H](O[C@H](CO)[C@@H](O)[C@@H]1OC(=O)CCCCCCCCCCCCCCCO)O[C@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1OS(O)(=O)=O', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@@H]1[C@@H](O)[C@H](O)[C@@H](CO)O[C@@H]1O[C@H]1O[C@H](CO)[C@@H](O)[C@H](O)[C@H]1OS([O-])(=O)=O', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('CCCCCCCCCCCCCCCC(=O)O[C@H]1[C@@H](O[C@H]2O[C@H](CO)[C@@H](O)[C@H](O)[C@H]2OS(O)(=O)=O)O[C@H](CO)[C@@H](O)[C@@H]1OC(=O)C(\\\\C)=C\\\\[C@@H](C)C[C@@H](C)C[C@@H](C)CCCCCCCC', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('[H][C@@]1(O[C@@](C[C@@H](O)[C@H]1O)(O[C@@H]1C[C@@](OC[C@H]2O[C@@H](OC[C@H]3O[C@H](OP(O)(O)=O)[C@H](NC(=O)C[C@H](O)CCCCCCCCCCC)[C@@H](OC(=O)C[C@H](O)CCCCCCCCCCC)[C@@H]3O)[C@H](NC(=O)C[C@@H](CCCCCCCCCCC)OC(=O)CCCCCCC\\\\C=C/CCCCCC)[C@@H](OC(=O)C[C@@H](CCCCCCCCCCC)OC(=O)CCCCCCCCCCCCC)[C@@H]2OP(O)(O)=O)(O[C@]([H])([C@H](O)CO)[C@@H]1O)C(O)=O)C(O)=O)[C@H](O)CO', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('[C@H]1(OP(=O)(O)O[C@H]2OC[C@@H]([C@@H]([C@H]2O)O)N)[C@H](O*)[C@@H](N*)[C@@H](O[C@@H]1CO)OC[C@@H]3[C@H]([C@@H]([C@H]([C@H](O3)OP(=O)(O)O)N*)O*)O', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('OC[C@@H](O)C(=C)C(=O)OC[C@H]1OC(O)[C@H](O)[C@@H](O)[C@@H]1O', "
               "'Insufficient hydroxyl groups for carbohydrate moiety'), "
               "('CCCCCCCCCCC[C@@H](O)CC(=O)N[C@H]1[C@H](OC[C@H]2O[C@H](OP(O)(O)=O)[C@H](NC(=O)C[C@H](O)CCCCCCCCCCC)[C@@H](OC(=O)C[C@H](O)CCCCCCCCCCC)[C@@H]2O)O[C@H](CO)[C@@H](O)[C@@H]1OC(=O)C[C@H](O)CCCCCCCCCCC', "
               "'Insufficient hydroxyl groups for carbohydrate moiety')]",
    'attempt': 1,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 8,
    'num_false_positives': 100,
    'num_true_negatives': 308,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.07407407407407407,
    'recall': 1.0,
    'f1': 0.13793103448275862,
    'accuracy': 0.7596153846153846}


"""
Classifies: CHEBI:17664 alkylaminium
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_alkylaminium(smiles: str):
    """
    Determines if a molecule is an alkylaminium, defined as a primary ammonium
    ion obtained by protonation of the amino group of any alkylamine; major species at pH 7.3.

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is an alkylaminium, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Find the ammonium nitrogen atom
    ammonium_n = None
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'N' and atom.GetFormalCharge() == 1:
            ammonium_n = atom
            break

    if ammonium_n is None:
        return False, "No ammonium nitrogen found"

    # Check if the ammonium nitrogen is part of a primary amine
    neighbors = ammonium_n.GetNeighbors()
    if len(neighbors) != 3:
        return False, "Ammonium nitrogen is not part of a primary amine"

    # Check if the substituents are alkyl groups
    alkyl_substituents = []
    for neighbor in neighbors:
        is_alkyl, alkyl_chain = is_alkyl_chain(mol, neighbor.GetIdx())
        if not is_alkyl:
            return False, f"Non-alkyl substituent found: {Chem.MolToSmiles(Chem.MolFromAtomBondAtomMapper(mol, neighbor.GetAtomicNum()))}"
        alkyl_substituents.append(alkyl_chain)

    return True, f"Alkylaminium with alkyl substituents: {', '.join(alkyl_substituents)}"

def is_alkyl_chain(mol, atom_idx):
    """
    Checks if the atom at the given index is part of an alkyl chain.

    Args:
        mol (Mol): RDKit molecule object
        atom_idx (int): Index of the atom to start from

    Returns:
        bool: True if the atom is part of an alkyl chain, False otherwise
        str: The alkyl chain in SMILES format
    """
    visited = set()
    queue = [(atom_idx, [])]

    while queue:
        curr_idx, chain = queue.pop(0)
        visited.add(curr_idx)
        atom = mol.GetAtomWithIdx(curr_idx)

        if atom.GetSymbol() != 'C':
            return False, ''

        neighbors = atom.GetNeighbors()
        if len(neighbors) > 2:
            return False, ''

        for neighbor in neighbors:
            if neighbor.GetIdx() not in visited:
                new_chain = chain + [neighbor.GetIdx()]
                queue.append((neighbor.GetIdx(), new_chain))

    return True, Chem.MolToSmiles(Chem.MolFromAtomBondAtomMapper(mol, [mol.GetAtomWithIdx(idx).GetAtomicNum() for idx in chain]))


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:17664',
                          'name': 'alkylaminium',
                          'definition': 'A primary ammonium ion obtained by '
                                        'protonation of the amino goup of any '
                                        'alkylamine; major species at pH 7.3.',
                          'parents': ['CHEBI:65296']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.8,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': False,
    'best': True,
    'error': "module 'rdkit.Chem' has no attribute 'MolFromAtomBondAtomMapper'",
    'stdout': '',
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 0,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': None}


"""
Classifies: CHEBI:23128 chloroalkane
"""
from rdkit import Chem
from rdkit.Chem import AllChem

def is_chloroalkane(smiles: str):
    """
    Determines if a molecule is a chloroalkane (alkane with at least one chlorine substituent).
    
    Args:
        smiles (str): SMILES string of the molecule
        
    Returns:
        bool: True if molecule is a chloroalkane, False otherwise
        str: Reason for classification
    """
    # Create RDKit molecule object
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return None, "Invalid SMILES string"

    # Check if molecule contains chlorine
    has_chlorine = False
    chlorine_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'Cl':
            has_chlorine = True
            chlorine_count += 1
    
    if not has_chlorine:
        return False, "No chlorine atoms found"

    # Check if molecule is an alkane (only sp3 carbons)
    is_alkane = True
    carbon_count = 0
    for atom in mol.GetAtoms():
        if atom.GetSymbol() == 'C':
            carbon_count += 1
            # Check hybridization - must be sp3 for alkanes
            if atom.GetHybridization() != Chem.HybridizationType.SP3:
                is_alkane = False
                break
        # Check for atoms other than C, H, Cl
        elif atom.GetSymbol() not in ['C', 'H', 'Cl']:
            is_alkane = False
            break

    if not is_alkane:
        return False, "Not an alkane (contains unsaturation or non C/H/Cl atoms)"
    
    if carbon_count == 0:
        return False, "No carbon atoms found"

    return True, f"Chloroalkane with {chlorine_count} chlorine atom(s)"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:23128',
                          'name': 'chloroalkane',
                          'definition': 'Any haloalkane that consists of an '
                                        'alkane substituted by at least one '
                                        'chloro group.',
                          'parents': ['CHEBI:24469', 'CHEBI:36683']},
    'config': {   'llm_model_name': 'lbl/claude-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 2,
    'num_false_positives': 14,
    'num_true_negatives': 183898,
    'num_false_negatives': 0,
    'num_negatives': None,
    'precision': 0.125,
    'recall': 1.0,
    'f1': 0.2222222222222222,
    'accuracy': 0.9999238774644671}


"""
Classifies: CHEBI:29348 fatty amide
"""
from rdkit import Chem
from rdkit.Chem import AllChem
from rdkit.Chem import Descriptors
from rdkit.Chem import rdMolDescriptors

def is_fatty_amide(smiles: str):
    """
    Determines if a molecule is a fatty amide (a monocarboxylic acid amide derived from a fatty acid).

    Args:
        smiles (str): SMILES string of the molecule

    Returns:
        bool: True if molecule is a fatty amide, False otherwise
        str: Reason for classification
    """
    mol = Chem.MolFromSmiles(smiles)
    if mol is None:
        return False, "Invalid SMILES string"

    # Check for amide functional group
    amide_atoms = [atom.GetIdx() for atom in mol.GetAtoms() if atom.GetSymbol() == 'N' and atom.GetTotalNumHs() == 0]
    for atom_idx in amide_atoms:
        atom = mol.GetAtomWithIdx(atom_idx)
        neighbors = [mol.GetAtomWithIdx(n.GetIdx()) for n in atom.GetNeighbors()]
        if len(neighbors) == 2 and neighbors[0].GetSymbol() == 'C' and neighbors[1].GetSymbol() == 'C':
            if neighbors[0].GetDegree() == 3 and neighbors[1].GetDegree() == 2:
                amide_carbonyl = neighbors[1]
                amide_carbon = neighbors[0]
            elif neighbors[1].GetDegree() == 3 and neighbors[0].GetDegree() == 2:
                amide_carbonyl = neighbors[0]
                amide_carbon = neighbors[1]
            else:
                continue

            # Check if amide is derived from a fatty acid
            fatty_acid_chain = []
            next_atom = amide_carbon
            while next_atom.GetDegree() == 2:
                fatty_acid_chain.append(next_atom.GetIdx())
                neighbors = [mol.GetAtomWithIdx(n.GetIdx()) for n in next_atom.GetNeighbors()]
                next_atom = neighbors[0] if neighbors[0].GetIdx() != fatty_acid_chain[-2] else neighbors[1]

            if len(fatty_acid_chain) >= 6:
                return True, f"Fatty amide derived from a {len(fatty_acid_chain) + 1}-carbon fatty acid"

    return False, "Not a fatty amide"


__metadata__ = {   'chemical_class': {   'id': 'CHEBI:29348',
                          'name': 'fatty amide',
                          'definition': 'A monocarboxylic acid amide derived '
                                        'from a fatty acid.',
                          'parents': ['CHEBI:29347', 'CHEBI:61697']},
    'config': {   'llm_model_name': 'claude-3-sonnet',
                  'f1_threshold': 0.0,
                  'max_attempts': 5,
                  'max_negative_to_test': None,
                  'max_positive_in_prompt': 50,
                  'max_negative_in_prompt': 20,
                  'max_instances_in_prompt': 100,
                  'test_proportion': 0.1},
    'message': None,
    'attempt': 0,
    'success': True,
    'best': True,
    'error': '',
    'stdout': None,
    'num_true_positives': 0,
    'num_false_positives': 0,
    'num_true_negatives': 183550,
    'num_false_negatives': 39,
    'num_negatives': None,
    'precision': 0.0,
    'recall': 0.0,
    'f1': 0.0,
    'accuracy': 0.9997875689719973}